package org.dcache.webdav;

import com.bradmcevoy.http.Response;
import com.bradmcevoy.http.Resource;

/**
 * Base class for WebDAV exceptions.
 *
 * These are RuntimExeptions to work around a limitation of
 * Milton. Many operations don't allow exceptions to be thrown. Hence
 * our only chance to report an error is to throw a
 * Runtimexception. However throwing RuntimException directly will
 * result in an "Internal Server Error" being reported back to the
 * client. Therefore we introduce the ErrorFilter which catches WebDAV
 * exceptions and responds with an appropriate error code.
 */
public class WebDavException extends RuntimeException
{
    private final Resource _resource;

    public WebDavException(Resource resource)
    {
        _resource = resource;
    }

    public WebDavException(String message, Resource resource)
    {
        super(message);
        _resource = resource;
    }

    public WebDavException(String message, Throwable cause, Resource resource)
    {
        super(message, cause);
        _resource = resource;
    }

    public Resource getResource()
    {
        return _resource;
    }
}